from collections import defaultdict


class Bidict(dict):
    def __init__(self, *args, **kwargs):
        super(Bidict, self).__init__(*args, **kwargs)

        self.inverse = defaultdict(list)
        for key, value in self.items():
            self.inverse[value].append(key)

    def __getitem__(self, key):
        value = self.inverse[key]
        if value:
            return value

        return super(Bidict, self).__getitem__(key)

    def __setitem__(self, key, value):
        super(Bidict, self).__setitem__(key, value)

        self.inverse[value].append(key)

    def __delitem__(self, key):
        super(Bidict, self).__delitem__(key)

        value = self[key]
        self.inverse[value].remove(key)
        if not self.inverse[value]:
            del self.inverse[value]
